//	Zinc Application Framework - Z_REAL1.CPP
//	COPYRIGHT (C) 1990-1998.  All Rights Reserved.
//	Zinc Software Incorporated.  Pleasant Grove, Utah  USA

#include <z_stdio.hpp>
#include <z_string.hpp>
#include <z_real1.hpp>
#include <z_error.hpp>
#include <z_app.hpp>
#define ZAF_REAL_INFO
#include "lang_def.cpp"
#include "gbl_def.cpp"

static ZafIChar numSeparator[3] = { '.','.',0 };
static ZafIChar rangeSeparator[3] = { ',',' ',0 };

// ----- ZafReal -----------------------------------------------------------

ZafReal::ZafReal(int left, int top, int width, ZafRealData *zRealData) :
	ZafString(left, top, width, new ZafStringData),
	realData(ZAF_NULLP(ZafRealData))
{
	// Get message strings.
	messages = ZafLanguageManager::Allocate(className, defaultMessages);

	// Initialize the real data.
	SetRealData(zRealData ? zRealData : new ZafRealData);
}

ZafReal::ZafReal(int left, int top, int width, double value) :
	ZafString(left, top, width, new ZafStringData),
	realData(ZAF_NULLP(ZafRealData))
{
	// Get message strings.
	messages = ZafLanguageManager::Allocate(className, defaultMessages);

	// Initialize the real data.
	SetRealData(new ZafRealData(value));
}

ZafReal::ZafReal(const ZafReal &copy) :
	ZafString(copy), realData(ZAF_NULLP(ZafRealData))
{
	// Get message strings.
	messages = ZafLanguageManager::Allocate(className, defaultMessages);

	// Copy the real data.
	if (copy.RealData()->Destroyable())
		SetRealData(new ZafRealData(*copy.RealData()));
	else
		SetRealData(copy.RealData());
}

ZafReal::~ZafReal(void)
{
	// Remove the data notification.
	realData->SubtractNotification(this, (ZafUpdateFunction)Update);

	// Free the messages.
	ZafLanguageManager::Free(messages);

	// Restore the real information.
	if (realData->Destroyable() && realData->NotifyCount() == 0)
		delete realData;
}

ZafError ZafReal::Decrement(ZafData *data, bool wrapData)
{
	// Check for a valid object.
	ZafRealData *real = DynamicPtrCast(data, ZafRealData);
	if (!real)
		return (ZAF_ERROR_INVALID_CLASS);

	// Decrement the value.
	ZafRealData _real(*realData);
	OSGetReal();
	*realData -= *real; // update is automatic.
	if (Validate(false) != ZAF_ERROR_NONE)
	{
		const ZafIChar *rangeText = RangeText();
		ZafIChar minNumber[ZAF_STRING_DATA_LENGTH];
		ZafIChar maxNumber[ZAF_STRING_DATA_LENGTH];
		if (wrapData && rangeText)
		{
			ParseRange(rangeText, minNumber, maxNumber);
			if (*maxNumber)
				realData->SetReal(maxNumber, OutputFormatText()); // update is automatic.
			else
				realData->SetReal(_real); // update is automatic.

		}
		else
			realData->SetReal(_real); // update is automatic.
		SetError(ZAF_ERROR_NONE);
	}

	// Return success.
	return (ZAF_ERROR_NONE);
}

ZafWindowObject *ZafReal::Duplicate(void)
{
	return (new ZafReal(*this));
}

ZafEventType ZafReal::Event(const ZafEventStruct &event)
{
	// Process OS specific messages.
	ZafEventType ccode = event.type;
	if (event.InputType() == E_KEY)
		ccode = LogicalEvent(event);

	// Check for zinc events.
	switch (ccode)
	{
	// ----- Create messages -----
	case S_INITIALIZE:
		OSSetReal();
		ccode = ZafString::Event(event);
		break;

	// ----- Action messages -----
	case L_SELECT:
	case N_NON_CURRENT:
		if (!error)
		{
			// Errors are reported only when focus changes within a window.
			bool leavingWindow = (ccode == N_NON_CURRENT && (!event.windowObject ||
				event.windowObject->RootObject() != RootObject())) ? true : false;

			// Keep a temporary number in case of error.
			ZafRealData oldValue(*realData);
			ZafError preValidateError = OSGetReal();

			// Report the error if not leaving the window.
			if (preValidateError != ZAF_ERROR_NONE && !leavingWindow)
			{
				ZafIChar *title = ZafLanguageManager::blankString;
				ZafIChar *errorString = messages->GetMessage(ZAF_ERROR_INVALID, true);
				zafErrorSystem->ReportError(windowManager, title, ZAF_DIALOG_OK, errorString, stringData->Text());
			}

			// See if the data has changed.
			newData = (*realData != oldValue || ((stringData->Length() == 0) != (unanswered == true))) ? true : false;

			// Call the user/validate function.
			ccode = ZafString::Event(event);

			// Check for errors.
			if (preValidateError != ZAF_ERROR_NONE)
				ccode = leavingWindow ? ZAF_ERROR_LEAVE_INVALID : preValidateError;
			if (ccode != 0)
			{
				if (ccode != ZAF_ERROR_LEAVE_INVALID)
					realData->SetReal(oldValue);
				else if (leavingWindow)
				{
					realData->SubtractNotification(this, (ZafUpdateFunction)Update);
					realData->SetReal(oldValue);
					realData->AddNotification(this, (ZafUpdateFunction)Update);
				}
			}
			else if (!unanswered)
				OSSetReal();

			// Clear the error code.
			SetError(ZAF_ERROR_NONE);
		}
		else
			// Nested call.
			return (ZAF_ERROR_LEAVE_INVALID);
		break;

	case S_COPY_DATA:
		{
		ZafWindowObject *object = event.windowObject;
		ZafReal *realNum = DynamicPtrCast(object, ZafReal);
		if (realNum)
			realData->SetReal(*realNum->RealData());
		}
		break;

	case S_SET_DATA:
		if (event.windowObject)
		{
			ZafWindowObject *object = event.windowObject;
			ZafReal *realNum = DynamicPtrCast(object, ZafReal);
			if (realNum)
				SetRealData(realNum->RealData());
		}
		else
			SetRealData(new ZafRealData(*realData));
		break;

	case N_RESET_I18N:
		OSSetReal();
		break;

	// ----- Default or unknown messages -----
	default:
		ccode = ZafString::Event(event);
		break;
	}

	// Return the control code.
	return (ccode);
}

ZafError ZafReal::Increment(ZafData *data, bool wrapData)
{
	// Check for a valid object.
	ZafRealData *real = DynamicPtrCast(data, ZafRealData);
	if (!real)
		return (ZAF_ERROR_INVALID_CLASS);

	// Increment the value.
	ZafRealData _real(*realData);
	OSGetReal();
	*realData += *real; // update is automatic.
	if (Validate(false) != ZAF_ERROR_NONE)
	{
		const ZafIChar *rangeText = RangeText();
		ZafIChar minNumber[ZAF_STRING_DATA_LENGTH];
		ZafIChar maxNumber[ZAF_STRING_DATA_LENGTH];
		if (wrapData && rangeText)
		{
			ParseRange(rangeText, minNumber, maxNumber);
			if (*minNumber)
				realData->SetReal(minNumber, OutputFormatText()); // update is automatic.
			else
				realData->SetReal(_real); // update is automatic.

		}
		else
			realData->SetReal(_real); // update is automatic.
		SetError(ZAF_ERROR_NONE);
	}

	// Return success.
	return (ZAF_ERROR_NONE);
}

bool ZafReal::IsA(ZafClassID compareID) const
{
	return ((compareID == classID) ? true : ZafString::IsA(compareID));
}

bool ZafReal::IsA(ZafClassName compareName) const
{
	return (!strcmp(compareName, className) ? true : ZafString::IsA(compareName));
}

ZafError ZafReal::Validate(bool processError)
{
	// Check for an absolute number error. Don't set the error member to ZAF_ERROR_NONE.
	ZafIChar *stringNumber = stringData->DynamicText();
	ZafRealData currentNumber;
	error = currentNumber.SetReal(stringNumber, InputFormatText());

	// Check for a range error.
	if (RangeText() && error == ZAF_ERROR_NONE)
		error = ZAF_ERROR_OUT_OF_RANGE;
	int numRanges = 0;
	ZafRealData low, high;
	ZafIChar minNumber[ZAF_STRING_DATA_LENGTH];
	ZafIChar maxNumber[ZAF_STRING_DATA_LENGTH];
	ZafIChar newNumber[ZAF_STRING_DATA_LENGTH];
	ZafIChar rBuffer[1024];				// Localized range string for error messages.
	rBuffer[0] = '\0';
	for (const ZafIChar *tRange = RangeText(); tRange && error == ZAF_ERROR_OUT_OF_RANGE; numRanges++)
	{
		tRange = ParseRange(tRange, minNumber, maxNumber);
		low.SetReal(minNumber, OutputFormatText());
		high.SetReal(maxNumber, OutputFormatText());
		if ((!minNumber[0] || currentNumber >= low) && (!maxNumber[0] || currentNumber <= high))
			error = ZAF_ERROR_NONE;
		else if (processError)
		{
			if (rBuffer[0])
				strcat(rBuffer, rangeSeparator);
			if (minNumber[0])
				low.FormattedText(&rBuffer[strlen(rBuffer)], ZAF_STRING_DATA_LENGTH, OutputFormatText());
			strcat(rBuffer, numSeparator);
			if (maxNumber[0])
				high.FormattedText(&rBuffer[strlen(rBuffer)], ZAF_STRING_DATA_LENGTH, OutputFormatText());
		}
	}

	// Process the error code.
	SetInvalid(false);
	if (error == ZAF_ERROR_NONE)
	{
		delete []stringNumber;
		return (error);
	}
	else
	{
		// Keep the error code.
		if (!zafErrorSystem || !processError || !numRanges)
		{
			// Restore the original number.
			realData->FormattedText(newNumber, ZAF_STRING_DATA_LENGTH, OutputFormatText());
			stringData->SetText(newNumber);
			delete []stringNumber;
			return (error);
		}
	}

	// Check for open-ended range errors.
	if (error == ZAF_ERROR_OUT_OF_RANGE && numRanges == 1)
	{
		if (minNumber[0] && !maxNumber[0])
		{
			error = ZAF_ERROR_LESS_THAN_RANGE;
			low.FormattedText(rBuffer, ZAF_STRING_DATA_LENGTH, OutputFormatText());
		}
		else if (!minNumber[0] && maxNumber[0])
		{
			error = ZAF_ERROR_GREATER_THAN_RANGE;
			high.FormattedText(rBuffer, ZAF_STRING_DATA_LENGTH, OutputFormatText());
		}
	}

	// Generate the error message and wait for a response.
	ZafIChar *title = ZafLanguageManager::blankString;
	ZafIChar *errorString = messages->GetMessage((ZafNumberID)error, true);
	if (errorString &&
		zafErrorSystem->ReportError(windowManager, title, allowInvalid ?
		ZAF_DIALOG_OK | ZAF_DIALOG_CANCEL : ZAF_DIALOG_CANCEL, errorString,
		stringNumber, rBuffer) == S_DLG_OK)
		error = ZAF_ERROR_LEAVE_INVALID; // Keep the new value.
	delete []stringNumber;
	return (error);
}

double ZafReal::Value(void)
{
	OSGetReal();
	return (RealData()->Value());
}

ZafError ZafReal::SetReal(double value)
{
	return (RealData()->SetReal(value));
}

ZafError ZafReal::SetRealData(ZafRealData *tRealData)
{
	// Remove the data notification.
	if (realData)
		realData->SubtractNotification(this, (ZafUpdateFunction)Update);

	// Reset the real data.
	if (realData && realData != tRealData &&
		realData->Destroyable() && realData->NotifyCount() == 0)
		delete realData;
	realData = tRealData ? tRealData : new ZafRealData;

	// Add the data notification.
	realData->AddNotification(this, (ZafUpdateFunction)Update);
	return (OSSetReal());
}

bool ZafReal::SetLowerCase(bool )
{
	// lowerCase is set to false by this class.
	lowerCase = false;
	return (lowerCase);
}

bool ZafReal::SetPassword(bool )
{
	// password is set to false by this class.
	password = false;
	return (password);
}

ZafError ZafReal::SetStringData(ZafStringData *)
{
	// String data cannot be set for this class.
	return (ZAF_ERROR_INVALID);
}

bool ZafReal::SetUpperCase(bool )
{
	// upperCase is set to false by this class.
	upperCase = false;
	return (upperCase);
}

bool ZafReal::SetVariableName(bool )
{
	// variableName is false for this class.
	variableName = false;
	return (variableName);
}

ZafError ZafReal::OSGetReal(void)
{
	// Disable notification.
	ZafUpdateType update = realData->Update(this);
	realData->SetUpdate(this, ZAF_UPDATE_NONE);

	// Set the real based on the string data.
	OSGetText();
	error = realData->SetReal(StringData()->Text(), InputFormatText());

	// Restore notification.
	realData->SetUpdate(this, update);
	return (error);
}

ZafError ZafReal::OSSetReal(void)
{
	if (unanswered)
		stringData->Clear();
	else
	{
		// Set the string based on the real data.
		ZafIChar text[ZAF_STRING_DATA_LENGTH];
		realData->FormattedText(text, ZAF_STRING_DATA_LENGTH, OutputFormatText());
		stringData->SetText(text);
	}

	// Return the current status.
	return (ZAF_ERROR_NONE);
}

// ----- Persistent functions -----------------------------------------------

ZafReal::ZafReal(const ZafIChar *name, ZafObjectPersistence &persist) :
	ZafString(name, persist.PushLevel(className, classID, ZAF_PERSIST_DIRECTORY)),
	realData(ZAF_NULLP(ZafRealData))
{
	// Get message strings.
	messages = ZafLanguageManager::Allocate(className, defaultMessages);

	if (persist.Error() == ZAF_ERROR_NONE)
		ZafReal::ReadData(persist);
	persist.PopLevel();
	if (persist.Error() != ZAF_ERROR_NONE)
		SetError(persist.Error());
}

ZafElement *ZafReal::Read(const ZafIChar *name, ZafObjectPersistence &persist)
{
	return (new ZafReal(name, persist));
}

void ZafReal::ReadData(ZafObjectPersistence &persist)
{
	// Read the data.
	ZafIChar realName[ZAF_MAXNAMELEN];
	ZafFile *file = persist.CurrentFile();
	*file >> realName;
	SetRealData(realName[0] ? new ZafRealData(realName, persist) : new ZafRealData);
}

void ZafReal::Write(ZafObjectPersistence &persist)
{
	// Write the object.
	ZafString::Write(persist.PushLevel(className, classID, ZAF_PERSIST_DIRECTORY));
	if (persist.Error() == ZAF_ERROR_NONE)
		ZafReal::WriteData(persist);
	persist.PopLevel();
	if (persist.Error() != ZAF_ERROR_NONE)
		SetError(persist.Error());
}

void ZafReal::WriteData(ZafObjectPersistence &persist)
{
	// Write the data.
	const ZafIChar *realName = realData ? realData->StringID() : ZAF_NULLP(ZafIChar);
	ZafFile *file = persist.CurrentFile();
	*file << realName;
	if (realData)
		realData->Write(persist);
}

